
# specify parameters

k <- 3
h <- 20
age <- 95
nboot <- 100
level <- 0.8

# demog.data[[5]] refers to Czech mortality rates

data <- demog.data[[5]]
data1 <- extract.ages(data, 0:age)
data2 <- extract.years(data1, 1950:2009)
data3 <- smooth.demogdata(data2)

f.smooth <- log(data3$rate$female)
m.smooth <- log(data3$rate$male)

uni.fore.obj.f <- uni.fore.obj.m <- array(NA, dim = c(h, age+1, h))
var.fore.obj.f <- var.fore.obj.m <- array(NA, c(h, age + 1, h))
vecm.fore.obj.f <- vecm.fore.obj.m <- array(NA, c(h, age+1, h))
pr.fore.obj.f <- pr.fore.obj.m <- array(NA, c(h, age+1, h))

var.boot.curve.f <- var.boot.curve.m <- array(NA, c(h, age + 1, h, nboot))
vecm.boot.curve.f <- vecm.boot.curve.m <- array(NA, c(h, age + 1, h, nboot))

uni.lb.f <- uni.ub.f <- uni.lb.m <- uni.ub.m <- array(NA, c(h, age+1, h))
pr.lb.f <- pr.ub.f <- pr.lb.m <- pr.ub.m <- array(NA, c(h, age+1, h))
vecm.lb.f <- vecm.ub.f <- vecm.lb.m <- vecm.ub.m <- array(NA, c(h, age+1, h))
var.lb.f <- var.ub.f <- var.lb.m <- var.ub.m <- array(NA, c(h, age+1, h))

for(iw in 1:h)
{
  data <- extract.years(data3, 1950:(1988 + iw))
  
  # UNI
  fts.obj.f <- fts(0:age, log(data$rate$female))
  fts.obj.m <- fts(0:age, log(data$rate$male))
  # default level is 80
  ftsm.fore.f <- forecast(ftsm(fts.obj.f, k), h = h, method="arima", level = 100*level)
  ftsm.fore.m <- forecast(ftsm(fts.obj.m, k), h = h, method="arima", level = 100*level)
  uni.fore.obj.f[iw,,] <- ftsm.fore.f$mean$y
  uni.fore.obj.m[iw,,] <- ftsm.fore.m$mean$y
  uni.lb.f[iw,,] <- ftsm.fore.f$lower$y
  uni.ub.f[iw,,] <- ftsm.fore.f$upper$y
  uni.lb.m[iw,,] <- ftsm.fore.m$lower$y
  uni.ub.m[iw,,] <- ftsm.fore.m$upper$y
  
  # VAR
  mod.var <- fvar(data, k)
  pred.var <- forecast.fvar(mod.var, h, level, nboot)
  var.fore.obj.f[iw,,] <- pred.var$Forecast$female
  var.fore.obj.m[iw,,] <- pred.var$Forecast$male
  var.boot.curve.f[iw,,,] <- pred.var$boot.sample$female
  var.boot.curve.m[iw,,,] <- pred.var$boot.sample$male
  var.lb.f[iw,,] <- pred.var$Intervals$female[, seq(1, ncol(pred.var$Intervals$female), 2)]
  var.ub.f[iw,,] <- pred.var$Intervals$female[, seq(2, ncol(pred.var$Intervals$female), 2)]
  var.lb.m[iw,,] <- pred.var$Intervals$male[, seq(1, ncol(pred.var$Intervals$male), 2)]
  var.ub.m[iw,,] <- pred.var$Intervals$male[, seq(2, ncol(pred.var$Intervals$male), 2)]
  
  # VECM
  mod.vecm <- fvecm(data, k)
  pred.vecm <- forecast.fvecm(mod.vecm, h, level, nboot)
  vecm.fore.obj.f[iw,,] <- pred.vecm$Forecast$female
  vecm.fore.obj.m[iw,,] <- pred.vecm$Forecast$male
  vecm.boot.curve.f[iw,,,] <- pred.vecm$boot.sample$female
  vecm.boot.curve.m[iw,,,] <- pred.vecm$boot.sample$male
  vecm.lb.f[iw,,] <- pred.vecm$Intervals$female[, seq(1, ncol(pred.vecm$Intervals$female), 2)]
  vecm.ub.f[iw,,] <- pred.vecm$Intervals$female[, seq(2, ncol(pred.vecm$Intervals$female), 2)]
  vecm.lb.m[iw,,] <- pred.vecm$Intervals$male[, seq(1, ncol(pred.vecm$Intervals$male), 2)]
  vecm.ub.m[iw,,] <- pred.vecm$Intervals$male[, seq(2, ncol(pred.vecm$Intervals$male), 2)]
  
  # PR
  pr.fit = coherentfdm(data)
  pr.fcast = forecast(pr.fit, h = h, level = 100*level)
  pr.fore.obj.f[iw,,] = log(pr.fcast$female$rate$female)
  pr.fore.obj.m[iw,,] = log(pr.fcast$male$rate$male)
  pr.lb.f[iw,,] = log(pr.fcast$female$rate$lower)
  pr.ub.f[iw,,] = log(pr.fcast$female$rate$upper)
  pr.lb.m[iw,,] = log(pr.fcast$male$rate$lower)
  pr.ub.m[iw,,] = log(pr.fcast$male$rate$upper)
  
}

# summarize results

uni.fore.val.f <- uni.fore.val.m <- var.fore.val.f <- var.fore.val.m <-
vecm.fore.val.f <- vecm.fore.val.m <- pr.fore.val.f <- pr.fore.val.m <- list()
for(iw in 1:h)
{
  uni.fore.val.f[[iw]] <- t(uni.fore.obj.f[1:(h+1-iw),,iw])
  uni.fore.val.m[[iw]] <- t(uni.fore.obj.m[1:(h+1-iw),,iw])
  var.fore.val.f[[iw]] <- t(var.fore.obj.f[1:(h+1-iw),,iw])
  var.fore.val.m[[iw]] <- t(var.fore.obj.m[1:(h+1-iw),,iw])
  
  vecm.fore.val.f[[iw]] <- t(vecm.fore.obj.f[1:(h+1-iw),,iw])
  vecm.fore.val.m[[iw]] <- t(vecm.fore.obj.m[1:(h+1-iw),,iw])
  pr.fore.val.f[[iw]] <- t(pr.fore.obj.f[1:(h+1-iw),,iw])
  pr.fore.val.m[[iw]] <- t(pr.fore.obj.m[1:(h+1-iw),,iw])
}

# if the mortality rates equal to 0 in the testing period, we set them to NA
data2$rate$female[data2$rate$female==0]=NA
data2$rate$male[data2$rate$male==0]=NA

# compute error measures in log scale for the four competing approaches

uni.f <- error(log(data2$rate$female[, 41:60]),  uni.fore.val.f , h)
uni.m <- error(log(data2$rate$male[, 41:60]),  uni.fore.val.m , h)

var.f <- error(log(data2$rate$female[, 41:60]),  var.fore.val.f , h)
var.m <- error(log(data2$rate$male[, 41:60]),  var.fore.val.m , h)

vecm.f <- error(log(data2$rate$female[, 41:60]), vecm.fore.val.f , h)
vecm.m <- error(log(data2$rate$male[, 41:60]),  vecm.fore.val.m , h)

pr.f <- error(log(data2$rate$female[, 41:60]),  pr.fore.val.f , h)
pr.m <- error(log(data2$rate$male[, 41:60]),  pr.fore.val.m , h)

# MAE and RME
#uni.f <- error(data2$rate$female[, 41:60], lapply(uni.fore.val.f, exp), h)
#uni.m <- error(data2$rate$male[, 41:60], lapply(uni.fore.val.m, exp), h)
#var.f <- error(data2$rate$female[, 41:60], lapply(var.fore.val.f, exp), h)
#var.m <- error(data2$rate$male[, 41:60], lapply(var.fore.val.m, exp), h)
#vecm.f <- error(data2$rate$female[, 41:60],lapply(vecm.fore.val.f, exp), h)
#vecm.m <- error(data2$rate$male[, 41:60], lapply(vecm.fore.val.m, exp), h)
#pr.f <- error(data2$rate$female[, 41:60], lapply(pr.fore.val.f, exp), h)
#pr.m <- error(data2$rate$male[, 41:60], lapply(pr.fore.val.m, exp), h)

## Interval scores

uni.lb.fore.f <- uni.ub.fore.f <- uni.lb.fore.m <- uni.ub.fore.m <-
var.lb.fore.f <- var.ub.fore.f <- var.lb.fore.m <- var.ub.fore.m <-
vecm.lb.fore.f <- vecm.ub.fore.f <- vecm.lb.fore.m <- vecm.ub.fore.m <-
pr.lb.fore.f <- pr.ub.fore.f <- pr.lb.fore.m <- pr.ub.fore.m <- list()

for(iw in 1:20)
{
  uni.lb.fore.f[[iw]] <- (t(uni.lb.f[1:(h+1-iw),,iw]))
  uni.ub.fore.f[[iw]] <- (t(uni.ub.f[1:(h+1-iw),,iw]))
  uni.lb.fore.m[[iw]] <- (t(uni.lb.m[1:(h+1-iw),,iw]))
  uni.ub.fore.m[[iw]] <- (t(uni.ub.m[1:(h+1-iw),,iw]))
  
  pr.lb.fore.f[[iw]] <- (t(pr.lb.f[1:(h+1-iw),,iw]))
  pr.ub.fore.f[[iw]] <- (t(pr.ub.f[1:(h+1-iw),,iw]))
  pr.lb.fore.m[[iw]] <- (t(pr.lb.m[1:(h+1-iw),,iw]))
  pr.ub.fore.m[[iw]] <- (t(pr.ub.m[1:(h+1-iw),,iw]))
  
  vecm.lb.fore.f[[iw]] <- (t(vecm.lb.f[1:(h+1-iw),,iw]))
  vecm.ub.fore.f[[iw]] <- (t(vecm.ub.f[1:(h+1-iw),,iw]))
  vecm.lb.fore.m[[iw]] <- (t(vecm.lb.m[1:(h+1-iw),,iw]))
  vecm.ub.fore.m[[iw]] <- (t(vecm.ub.m[1:(h+1-iw),,iw]))
  
  var.lb.fore.f[[iw]] <- (t(var.lb.f[1:(h+1-iw),,iw]))
  var.ub.fore.f[[iw]] <- (t(var.ub.f[1:(h+1-iw),,iw]))
  var.lb.fore.m[[iw]] <- (t(var.lb.m[1:(h+1-iw),,iw]))
  var.ub.fore.m[[iw]] <- (t(var.ub.m[1:(h+1-iw),,iw]))
} 

#uni.score.f <- int.score(data2$rate$female[, 41:60], lapply(uni.lb.fore.f, exp), lapply(uni.ub.fore.f, exp), h, level)
#uni.score.m <- int.score(data2$rate$male[, 41:60], lapply(uni.lb.fore.m, exp), lapply(uni.ub.fore.m, exp), h, level)
#var.score.f <- int.score(data2$rate$female[, 41:60], lapply(var.lb.fore.f, exp), lapply(var.ub.fore.f, exp), h, level)
#var.score.m <- int.score(data2$rate$male[, 41:60], lapply(var.lb.fore.m, exp), lapply(var.ub.fore.m, exp), h, level)
#vecm.score.f <- int.score(data2$rate$female[, 41:60], lapply(vecm.lb.fore.f, exp), lapply(vecm.ub.fore.f, exp), h, level)
#vecm.score.m <- int.score(data2$rate$male[, 41:60], lapply(vecm.lb.fore.m, exp), lapply(vecm.ub.fore.m, exp), h, level)
#pr.score.f <- int.score(data2$rate$female[, 41:60], lapply(pr.lb.fore.f, exp), lapply(pr.ub.fore.f, exp), h, level)
#pr.score.m <- int.score(data2$rate$male[, 41:60], lapply(pr.lb.fore.m, exp), lapply(pr.ub.fore.m, exp), h, level)

uni.score.f <- int.score(log(data2$rate$female[, 41:60]),  uni.lb.fore.f ,  uni.ub.fore.f , h, level)
uni.score.m <- int.score(log(data2$rate$male[, 41:60]),  uni.lb.fore.m ,  uni.ub.fore.m , h, level)
var.score.f <- int.score(log(data2$rate$female[, 41:60]),  var.lb.fore.f ,  var.ub.fore.f , h, level)
var.score.m <- int.score(log(data2$rate$male[, 41:60]),  var.lb.fore.m ,  var.ub.fore.m , h, level)
vecm.score.f <- int.score(log(data2$rate$female[, 41:60]),  vecm.lb.fore.f ,  vecm.ub.fore.f , h, level)
vecm.score.m <- int.score(log(data2$rate$male[, 41:60]),  vecm.lb.fore.m ,  vecm.ub.fore.m , h, level)
pr.score.f <- int.score(log(data2$rate$female[, 41:60]),  pr.lb.fore.f ,  pr.ub.fore.f , h, level)
pr.score.m <- int.score(log(data2$rate$male[, 41:60]),  pr.lb.fore.m ,  pr.ub.fore.m , h, level)

# plotting results

plot(uni.f$mae, type = "l")
lines(var.f$mae, col = 2)
lines(vecm.f$mae, col = 3)
lines(pr.f$mae, col = 4)

plot(uni.m$mae, type = "l")
lines(var.m$mae, col = 2)
lines(vecm.m$mae, col = 3)
lines(pr.m$mae, col = 4)

plot(uni.f$mse, type = "l")
lines(var.f$mse, col = 2)
lines(vecm.f$mse, col = 3)
lines(pr.f$mse, col = 4)

plot(uni.m$mse, type = "l")
lines(var.m$mse, col = 2)
lines(vecm.m$mse, col = 3)
lines(pr.m$mse, col = 4)

plot(uni.score.f, type = "l")
lines(var.score.f, col = 2)
lines(vecm.score.f, col = 3)
lines(pr.score.f, col = 4)

plot(uni.score.m, type = "l")
lines(var.score.m, col = 2)
lines(vecm.score.m, col = 3)
lines(pr.score.m, col = 4)

