###################
# load R packages
###################

install.packages("VAR.etp")
install.packages("demography")
install.packages("vars")
install.packages("ftsa")
install.packages("tsDyn")
install.packages("mnormt")

library(VAR.etp)
library(demography)
library(vars)
library(ftsa)
library(tsDyn)
library(mnormt)

#########################################
# mae and mse function with removing NAs
#########################################

mae <- function(forecast, true)
{
  if (length(forecast) != length(true))
    stop("MAE: the lengths of input vectors must be the same.")
  err <- mean(abs(true - forecast), na.rm = T)
  return(round(err, 6))
}

mse <- function(forecast, true)
{
  if (length(forecast) != length(true))
    stop("RMAE: the lengths of input vectors must be the same.")
  err <- mean((true - forecast)^2, na.rm = T)
  return(round(err, 6))
}

#############################
# interval score function
#############################

int.score <- function(x, lb, ub, h = 20, alpha = 0.8)
{
  score <- vector(,h)
  for(iw in 1:h)
  {
    lb.ind <- ifelse(x[,iw:h] < lb[[iw]], 1, 0)
    ub.ind <- ifelse(x[,iw:h] > ub[[iw]], 1, 0)
    score[iw] <- mean((ub[[iw]] - lb[[iw]]) + 2/(1-alpha) * (lb[[iw]] - x[,iw:h]) * lb.ind + 
                       2/(1-alpha) * (x[,iw:h] - ub[[iw]]) * ub.ind, na.rm = T)
  }
  return(score) 
}

#############################
# error function
#############################

error <- function(x, x.fore, h)
{
  mae <- vector(,h)
  mse <- vector(,h)
  for(iw in 1:h)
  {
    mae[iw] <- mae(x.fore[[iw]], x[,iw:h])
    mse[iw] <- mse(x.fore[[iw]], x[,iw:h])
  }
  return(list(mae = mae, mse = mse))
}


##########################################
# read data from Human Mortality Database
##########################################

demog.data <- list()
demog.data[[1]] <- hmd.mx("AUS", "gaofield@hotmail.com", 19911122, label="Australia")
demog.data[[2]] <- hmd.mx("AUT", "gaofield@hotmail.com", 19911122, label="Austria")
demog.data[[3]] <- hmd.mx("BGR", "gaofield@hotmail.com", 19911122, label="Bulgaria")
demog.data[[4]] <- hmd.mx("CAN", "gaofield@hotmail.com", 19911122, label="Canada")
demog.data[[5]] <- hmd.mx("CZE", "gaofield@hotmail.com", 19911122, label="Czech")
demog.data[[6]] <- hmd.mx("DNK", "gaofield@hotmail.com", 19911122, label="Danmark")
demog.data[[7]] <- hmd.mx("FIN", "gaofield@hotmail.com", 19911122, label="Finland")
demog.data[[8]] <- hmd.mx("FRATNP", "gaofield@hotmail.com", 19911122, label="France")
demog.data[[9]] <- hmd.mx("HUN", "gaofield@hotmail.com", 19911122, label="Hungary")
demog.data[[10]] <- hmd.mx("ISL", "gaofield@hotmail.com", 19911122, label="Iceland")
demog.data[[11]] <- hmd.mx("ITA", "gaofield@hotmail.com", 19911122, label="Italy")
demog.data[[12]] <- hmd.mx("JPN", "gaofield@hotmail.com", 19911122, label="Japan")
demog.data[[13]] <- hmd.mx("NLD", "gaofield@hotmail.com", 19911122, label="Netherlands")
demog.data[[14]] <- hmd.mx("NOR", "gaofield@hotmail.com", 19911122, label="Norway")
demog.data[[15]] <- hmd.mx("PRT", "gaofield@hotmail.com", 19911122, label="Portugal")
demog.data[[16]] <- hmd.mx("ESP", "gaofield@hotmail.com", 19911122, label="Spain")
demog.data[[17]] <- hmd.mx("SWE", "gaofield@hotmail.com", 19911122, label="Sweden")
demog.data[[18]] <- hmd.mx("CHE", "gaofield@hotmail.com", 19911122, label="Switzerland")
demog.data[[19]] <- hmd.mx("GBR_NP", "gaofield@hotmail.com", 19911122, label="UK")
demog.data[[20]] <- hmd.mx("USA", "gaofield@hotmail.com", 19911122, label="USA")

fix(smooth.demogdata)
# ADD
data$smooth.err$female <- (1-data$rate$female)/data$rate$female/data$pop$female
data$smooth.err$male <- (1-data$rate$male)/data$rate$male/data$pop$male

